local aceHook = LibStub("AceHook-3.0")
local aceEvent = LibStub("AceEvent-3.0")
local _G = getfenv(0)
local format = _G.format
local UnitIsPlayer = _G.UnitIsPlayer
local UnitCanAttack = _G.UnitCanAttack
local UnitIsPVP = _G.UnitIsPVP
local UnitClass = _G.UnitClass
local UnitPlayerControlled = _G.UnitPlayerControlled
local UnitExists = _G.UnitExists
local UnitName = _G.UnitName
local GetGuildInfo = _G.GetGuildInfo
local UnitIsUnit = _G.UnitIsUnit
local UnitCreatureFamily = _G.UnitCreatureFamily
local UnitCreatureType = _G.UnitCreatureType
local UnitMana = _G.UnitMana
local UnitManaMax = _G.UnitManaMax
local UnitIsTapped = _G.UnitIsTapped
local UnitIsTappedByPlayer = _G.UnitIsTappedByPlayer

local friendlist = {}

CreateFrame("GameTooltip", "tehtip", UIParent, "GameTooltipTemplate")
tehtip:SetOwner(WorldFrame, "ANCHOR_NONE")

GameTooltipStatusBar:SetStatusBarTexture("Interface\\AddOns\\!Tehtip\\Minimalist.tga")
local ManaBar = CreateFrame("StatusBar", nil, GameTooltipStatusBar)
ManaBar:SetStatusBarTexture("Interface\\AddOns\\!Tehtip\\Minimalist.tga")
ManaBar:SetHeight(8)
ManaBar:SetPoint("TOPLEFT", GameTooltip, "BOTTOMLEFT", 2, -10)
ManaBar:SetPoint("TOPRIGHT", GameTooltip, "BOTTOMRIGHT", -2, -10)

local classColours = {}
local factionColors = {}
for k,v in pairs(CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS) do
	classColours[k] = format("%2x%2x%2x", v.r*255, v.g*255, v.b*255)
end
for k,v in pairs(FACTION_BAR_COLORS) do
	factionColors[k] = format("%2x%2x%2x", v.r*255, v.g*255, v.b*255)
end

local function find(text, pattern)
	local start, stop = string.find(text, pattern)
	if start and stop then return string.sub(text, start, stop) end
end

local LEVEL_start = "^" .. (type(LEVEL) == "string" and LEVEL or "Level")
local function NPCSubtitle(unit)
	tehtip:ClearLines()
	tehtip:SetUnit(unit)
	if not tehtip:IsOwned(UIParent) then
		tehtip:SetOwner(UIParent, "ANCHOR_NONE")
	end
	local left_2 = tehtipTextLeft2:GetText()
	if not left_2 or find(left_2, LEVEL_start) then
		return nil
	end
	return left_2
end
local PVP = type(PVP) == "string" and PVP or "PvP"
local function NPCFaction(unit)
	tehtip:ClearLines()
	tehtip:SetUnit(unit)
	if not tehtip:IsOwned(UIParent) then
		tehtip:SetOwner(UIParent, "ANCHOR_NONE")
	end
	local left_2 = tehtipTextLeft2:GetText()
	local left_3 = tehtipTextLeft3:GetText()
	
	local hasGuild = not find(left_2, LEVEL_start)
	local factionText = not hasGuild and left_3 or tehtipTextLeft4:GetText()
	if factionText == PVP then
		return faction
	else
		return factionText
	end
end

local function updateAnchor()
	GameTooltip:ClearAllPoints()
	local scale = GameTooltip:GetEffectiveScale()
	local x, y = GetCursorPosition()
	x, y = x / scale + 55, y / scale - 100
	GameTooltip:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", x, y)
end

local function colorUnit(unit)
	local color
	if UnitIsPlayer(unit) and not UnitCanAttack("player", unit) and not UnitCanAttack(unit, "player") then
		if UnitIsPVP(unit) then
			color = factionColors[5]
		else
			local class = select(2, UnitClass(unit))
			color = classColours[class]
		end
	else
		local reaction = UnitReaction(unit, "player")
		if UnitPlayerControlled(unit) and reaction >= 5 then
			color = "00ABFF"
		else
			color = factionColors[reaction]
		end
	end
	return color or "FFFFFF"
end

function aceHook:formatTooltip(self)
	GameTooltipStatusBar:SetStatusBarColor(0, 0, 1)
	--add the lines!
	local line = 1
	local unitName, unit = self:GetUnit()
	--print(unit)
	if UnitExists("mouseover") or (UnitExists("mouseover") and not UnitInRange("mouseover")) then
		local target
		local targetText
		local npcSubtitle
		local guild
		local factionText
		local unitColor = colorUnit(unit)
		local manaMax = UnitManaMax(unit)
		local mana = UnitMana(unit)
		ManaBar:SetMinMaxValues(0, manaMax)
		ManaBar:SetValue(mana)
		ManaBar:Show()
		-- ninjad from Cowtip
		local powerType = UnitPowerType(unit)
		if powerType == 0 then
			-- mana
			ManaBar:SetStatusBarColor(48/255, 113/255, 191/255)
		elseif powerType == 1 then
			-- rage
			ManaBar:SetStatusBarColor(226/255, 45/255, 75/255)
		elseif powerType == 2 then
			-- focus
			ManaBar:SetStatusBarColor(1, 210/255, 0)
		elseif powerType == 6 then
			-- runic power
			ManaBar:SetStatusBarColor(0.00, 0.82, 1.00)
		else
			-- energy
			ManaBar:SetStatusBarColor(1, 220/255, 25/255)
		end
		
		if unit then target = UnitName(format("%s%s", unit, "target")) end
		if target then
			if UnitIsUnit(target, "player") then
				targetText = format(":%s", "|cFFFFFFFF<<YOU>>|r")
			else
				local color = colorUnit("mouseovertarget")
				targetText = format(":|cFF%s%s|r", color, target)
			end
		else
			targetText = ""
		end
		
		local unitText = format("|cFF%s%s|r%s", unitColor, unitName, targetText)
		
		local raceclass
		-- NPC!
		if not UnitIsPlayer(unit) and not UnitPlayerControlled(unit) then
			local title = NPCSubtitle(unit)
			local color = "00ABFF"
			if title then npcSubtitle = format("|cFF%s<%s>|r", color, title) end
			
			raceclass = format(" |cFFE5B200%s|r", UnitCreatureType(unit))
			factionText = NPCFaction(unit)
		-- Pet!
		elseif (not UnitIsPlayer(unit) and UnitPlayerControlled(unit)) then
			local title = NPCSubtitle(unit)
			local color = "00ABFF"
			if title then npcSubtitle = format("|cFF%s<%s>|r", color, title) end
			
			local family = UnitCreatureFamily(unit)
			local creatureType = UnitCreatureType(unit)
			raceclass = format(" |cFFE5B200%s|r", family or creatureType)
		-- Player!
		else
			local g = GetGuildInfo(unit)
			local color
			if UnitIsInMyGuild(unit) or friendlist[unitName] then
				color = "9400D4"
			else
				color = "00ABFF"
			end
			if g then guild = format("|cFF%s<%s>", color, g) end
			
			local race = UnitRace(unit)
			local eClass, class = UnitClass(unit)
			color = classColours[class] or "CC4C38"
			local coloredClass = format("|cFF%s%s|r", color, eClass)
			raceclass = format(" |cFFE5B200%s|r %s", race, coloredClass)
		end
		local level = UnitLevel(unit)
		if level == -1 then level = "??" end
		local levelLine = format("%s%s", level, raceclass)
		
		if unitText then
			_G["GameTooltipTextLeft"..1]:SetText(unitText)
			line = line + 1
		end
		if npcSubtitle then
			_G["GameTooltipTextLeft"..line]:SetText(npcSubtitle)
			line = line + 1
		end
		if guild then
			_G["GameTooltipTextLeft"..line]:SetText(guild)
			line = line + 1
		end
		if levelLine then
			_G["GameTooltipTextLeft"..line]:SetText(levelLine)
			line = line + 1
		end
		if factionText then
			_G["GameTooltipTextLeft"..line]:SetText(factionText)
			line = line + 1
		end
		
		-- background coloring
		local bR, bG, bB
		if (UnitIsDead(unit) and not UnitIsFeignDeath(unit)) or (UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit)) then
			bR, bG, bB = 0.54, 0.54, 0.54
		elseif not UnitIsPlayer(unit) and not UnitPlayerControlled(unit) then
			local reaction = UnitReaction(unit, "player")
			bR, bG, bB = FACTION_BAR_COLORS[reaction].r, FACTION_BAR_COLORS[reaction].g, FACTION_BAR_COLORS[reaction].b
		else
			bR, bG, bB = 0.0, 0.0, 0.5
		end
		
		
		GameTooltipTextLeft1:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
		GameTooltip:SetBackdropColor(bR, bG, bB)
		GameTooltip:Show()
	end
end

local TipUpdateFrame = CreateFrame("Frame")
local AnchorUpdateFrame = CreateFrame("Frame")

local function updateTip()
	aceHook:formatTooltip(GameTooltip)
end
local function scriptUpdateTip()
	TipUpdateFrame:SetScript("OnUpdate", updateTip)
end
local function resetScripts()
	TipUpdateFrame:SetScript("OnUpdate", nil)
	AnchorUpdateFrame:SetScript("OnUpdate", nil)
end

local function SetDefaultAnchor(tooltip, owner)
	if owner ~= UIParent then
		AnchorUpdateFrame:SetScript("OnUpdate", nil)
	else
		AnchorUpdateFrame:SetScript("OnUpdate", updateAnchor)
	end
end

aceHook:SecureHook("GameTooltip_SetDefaultAnchor", SetDefaultAnchor)
aceHook:SecureHookScript(GameTooltip, "OnTooltipSetUnit", scriptUpdateTip)
aceHook:SecureHookScript(GameTooltip, "OnTooltipCleared", resetScripts)

local function FRIENDLIST_UPDATE()
	friendlist = {}
	for i=1, GetNumFriends() do
		local name = GetFriendInfo(i)
		friendlist[name] = name
	end
end
FRIENDLIST_UPDATE()

TipUpdateFrame:RegisterEvent("FRIENDLIST_UPDATE")
TipUpdateFrame:SetScript("OnEvent", FRIENDLIST_UPDATE)
